/**
 * 基本数据类型 
 */

#ifndef __XC_BASE_TYPES_H
#define __XC_BASE_TYPES_H

#include <netinet/in.h>

/* IP 地址定义 */
union ip_addr {
    struct in_addr ipv4; // IPv4 地址
    struct in6_addr ipv6; // IPv6 地址
};

/* 说明： 若要表示任意 IP 地址时，IPv4 是 0.0.0.0 ， IPv6 是 ::0 */
/* IP 地址范围 */
struct ip_range {
    union ip_addr start; // 范围起点
    union ip_addr end; // 范围终点
};

/* 端口范围 */
struct port_range {
    uint16_t start; // 范围起点
    uint16_t end; // 范围终点
};

/* IP 地址版本定义 */
typedef enum {
    IP_VERSION_IPV4 = 0,
    IP_VERSION_IPV6,
    IP_VERSION_SIZE,
} ip_version_t;

/* 网络五元组结构体定义 */
struct net_five_tuple {
    struct ip_range src_ip_range; // 源 IP 地址范围，为固定值时，start,end 为相同的值
    struct ip_range dst_ip_range; // 目标 IP 地址范围，为固定值时，start,end 为相同的值
    struct port_range src_port_range; // 源端口范围，为固定值时，start,end 为相同的值
    struct port_range dst_port_range; // 目标端口范围，为固定值时，start,end 为相同的值
    uint8_t protocol; // 协议类型
    ip_version_t ip_version; // IP 协议版本
};

/* 网络流量的出站和入站方向 */
typedef enum {
    NETFLOW_INBOUND = 0 , // 入站网络流量
    NETFLOW_OUTBOUND, // 出站网络流量
    NETFLOW_ALL, // 所有流量
    NETFLOW_TYPE_SIZE,
} netflow_dir_t;

/**
 * 协议类型 
 */
enum protocol_type
{
    PROTOCOL_TCP = 0,
    PROTOCOL_UDP,
    PROTOCOL_TYPE_SIZE,
};

enum policy_type {
    NETFLOW_PROCESS = 1,
    NETFLOW_TUPLE,
};

/* 网络流量审计策略 */
struct netflow_audit_policy
{ 
    uint16_t policy_id; //策略编号
    char process_name[16]; //进程名称
    struct net_five_tuple tuple; //五元组信息
    netflow_dir_t direction; //网络流量方向
    int type; // 策略的类型，1. 进程，2.五元组
};

/* 接口类型 */
enum opt_type
{
    audit_add_policy = 0,                   // 添加网络审计策略
    audit_delete_policy,                    // 删除网络审计策略
    audit_delete_all,                       // 清空网络审计策略
    audit_policy_counts,                    // 获取所有网络流量审计策略
    audit_socket_open,                      // 获取进程审计流量数据信息的socket句柄的接口
    audit_socket_close,                     // 关闭进程审计流量数据信息句柄接口    
    drop_add_policy,                        // 添加网络数据阻断策略
    drop_delete_policy,                     // 删除网络数据阻断策略
    drop_delete_all,                        // 清空网络数据阻断策略
    drop_policy_counts,                     // 获取所有网络流量阻断策略
    drop_socket_open,                       // 获取进程阻断流量数据信息的socket句柄的接口
    drop_socket_close,                      // 关闭进程阻断流量数据信息句柄接口 
    audit_add_proc_policy,                  // 添加进程审计策略 
    audit_add_proc_policy_complete,         // 添加进程策略完成
    audit_add_proc_policy_heartbeat,        // 进程审计服务心跳
};

/* 网络流量审计数据包 */
struct netflow_audit_pkt {
    pid_t pid;                              //数据包所属进程 pid
    char process_name[16];                  //数据包所属进程名
    uint16_t policy_id;               //策略编号
    struct net_five_tuple tuple;            //五元组信息
    int pkt_len;                            //数据包长度
    char payload[256];                      //数据包审计负载,只截取每个包的前面256 字节 
    netflow_dir_t direction;                //网络流量方向
};

// 名称 网络流量阻断日志内容定义
struct netflow_drop_log {
    struct net_five_tuple tuple_info; // 五元组信息
    time_t timestamp; //时间戳
    pid_t pid; //进程编号
    char process_name[16]; //进程名称
    uint16_t policy_id; //策略编号
    netflow_dir_t direction; //网络流量方向
    int pkt_len; //数据包长度
};

// 名称 网络流量阻断策略数据结构定义
struct netflow_drop_policy {
    uint16_t policy_id;                     //策略编号
    struct net_five_tuple tuple;            // 网络五元组信息
    netflow_dir_t direction;                // 要管控的网络流量方向
};

// 进程审计策略
struct netflow_audit_proc_policy
{
    uint16_t policy_id;             // 策略编号
    uint16_t port;                  // 进程端口
    uint8_t protocol;               // 协议类型
    netflow_dir_t direction;        // 要管控的网络流量方向
    pid_t pid;                      // 策略所属进程 pid
    char process_name[16];          // 策略所属进程名
};

/* 网络操作参数 */
union net_opt_param
{
    struct netflow_audit_policy audit_policy;       // 网络流量审计策略
    uint16_t audit_policy_id;                       // 删除网络数据审计策略时使用
    uint16_t audit_policy_all;                      // 清空网络数据审计策略时使用
    struct netflow_drop_policy  drop_policy;        // 添加网络数据阻断策略时使用
    uint16_t drop_policy_id;                        // 删除网络数据阻断策略时使用
    uint16_t drop_policy_all;                       // 清空网络数据阻断策略时使用
    struct netflow_audit_proc_policy audit_proc_policy; // 进程审计策略
};

/* 统一参数 */
struct unified_param
{
    enum opt_type type;                     // 接口类型
    union net_opt_param param;              // 接口参数
};

// 策略表ID
struct policy_table_id
{
    int protocol;       // 协议
    int direction;      // 方向
    int ip_version;     // IP版本
};

#endif
